# Author::    Maurizio Casimirri (mailto:maurizio.cas@gmail.com)
# Copyright:: Copyright (c) 2012 Maurizio Casimirri
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

require 'rails'

module ResourcePresentationHelpers

  def image_method_for(resource, opts = {})
    klass = resource.is_a?(Class) ? resource : resource.class
    any_names = ((klass.respond_to?(:column_names)  ? klass.column_names : []) + klass.public_instance_methods).map {|name| "#{name}"}.uniq.delete_if {|meth| meth =~ /^_/}
    %w(photo image thumbnail logo).find {|meth| any_names.include?(meth)}
  end
      
  def image_for(resource, opts = {})
    attachment_or_url = resource && (meth = image_method_for(resource, opts)) && resource.send(meth)
    return nil if attachment_or_url.nil?

    if attachment_or_url.is_a?(Paperclip::Attachment)
      style = opts[:style] && attachment_or_url.styles.keys.include?("#{opts[:style]}") ? opts[:style] : :original 
      attachment_or_url.send(:url, style)
    else
      attachment_or_url        
    end
  end
    
  def name_method_for(resource, opts = {})
    klass = resource.is_a?(Class) ? resource : resource.class
    any_names = ((klass.respond_to?(:column_names)  ? klass.column_names : []) + klass.public_instance_methods).map {|name| "#{name}"}.uniq.delete_if {|meth| meth =~ /^_/}
    %w(to_label title name label browser_title seo_title seo_name key claim email to_s).find {|meth| any_names.include?(meth)}
  end
  alias :title_method_for :name_method_for 
  alias :label_method_for :name_method_for

  def name_for(resource, opts = {})
    resource && (meth = label_method_for(resource, opts)) && resource.send(meth)
  end
  alias :title_for :name_for 
  alias :label_for :name_for

  def description_method_for(resource, opts = {})
    klass = resource.is_a?(Class) ? resource : resource.class
    any_names = ((klass.respond_to?(:column_names)  ? klass.column_names : []) + klass.public_instance_methods).map {|name| "#{name}"}.uniq.delete_if {|meth| meth =~ /^_/}
    %w(description excerpt meta_descripiton to_description).find {|meth| any_names.include?(meth)}
  end

  def description_for(resource, opts = {})
    resource && ( meth = description_method_for(resource, opts) ) && resource.send(meth)
  end

end

ActionView::Base.send :include, ResourcePresentationHelpers


